require "pgkingdom/version"
require "pgkingdom/table"

# Provides global namespace and methods.
module PgKingdom
  class << self
    # Filters :todo, :unsupported, etc from {.data_types} and returns flattened.
    # @return [Array<Symbol>] all supported data types
    def available_data_types
      @_available_data_types ||= data_types.select do |key, val|
        ![:TODO, :DONT_USE_ARRAYS_EXPLICITLY, :NOT_SUPPORTED, []].include? val
      end.map { |key, val| val }.flatten
    end

    # Returns a Hash of PostgreSQL data types.
    # @return [Hash{String=>(Symbol|Array<Symbol>|Array<Array<Symbol>>)})] contains all available PostgreSQL data types
    # @see https://www.postgresql.org/docs/9.6/static/datatype.html
    def data_types
      @_data_types ||= {
        arrays: [ :DONT_USE_ARRAYS_EXPLICITLY ], # Included as option :array => true into primitive types
        uuid: [ :uuid ], # REQUIRES: CREATE EXTENSION "uuid-ossp";
        binary: [ :bytea ],
        boolean: [ :boolean ],
        monetary: [ :money ],
        bit_string: [ :bit_verying, :bit ],
        network_address: [ :cidr, :inet, :macaddr ],
        numeric: [
          [:smallint, :integer, :bigint], #integer
          [:decimal, :numeric, :real, :double_precision], # floating point
          [:smallserial, :serial, :bigserial] # serials
        ],
        character: [
          [:varchar, :character_varying],
          [:char, :character],
          [:text]
        ],
        datetime: [
          [:timestamp, :timestamp_without_tz, :timestamp_with_tz],
          [:date],
          [:time, :time_without_tz, :time_with_tz],
          [:interval]
        ],
        range: [
          [:int4range, :int8range, :numrange],
          [:tsrange, :tstzrange, :daterange]
        ],
        enumerated: [
          :TODO
        ],
        text_search: [
          :TODO
        ],
        json: [
          :TODO
        ],
        oid: [ :NOT_SUPPORTED ],
        xml: [ :NOT_SUPPORTED ],
        pg_lsn: [ :NOT_SUPPORTED ],
        pseudo: [ :NOT_SUPPORTED ],
        geometric: [ :NOT_SUPPORTED ],
        composite: [ :NOT_SUPPORTED ],
      }
    end
  end
end
