require 'pgkingdom/options_helper'

module PgKingdom
  # Internally used to extend Arrays of columns in tables.
  module ColumnsHelper
    # Joins columns for a statement.
    # @return [String] columns definition with options.
    def join
      map do |col|
        opt_string = ""

        # Flags
        options = col[:options]
        options.extend OptionsHelper

        "#{col[:name]} #{col[:type].upcase}#{options.join}"
      end.join(",\n\s\s")
    end

    # Checks if provided type is valid PostgreSQL data type.
    def available?(type)
      PgKingdom.available_data_types.include? type
    end

    # Adds column definition hash to array.
    # @return [Array<Hash{Symbol=>(Numeric|Symbol|String|Hash)}>] Array with Hashes describing table columns.
    def add(name, type, opts)
      self << { :name => name, :type => type, :options => opts }
    end

    # Checks if column defined in table.
    def defined?(name)
      select { |a| a[:name] == name }.any?
    end
  end
end
