module PgKingdom
  # Internally used to extend Arrays of column options in tables.
  module OptionsHelper
    # Joins options for a column.
    # @return [String] column options.
    def join
      options = clone
      numerics = []
      result = options.map do |option|
        case option
        when Numeric
          numerics << option
          nil
        when Hash
          option
            .map { |opt, value| option_to_sql opt, value }
            .join ' '
        else
          option_to_sql option
        end
      end

      result.reject! { |opt| opt.nil? }
      result = result.join(' ')
      numerics.any? ? "(#{numerics.join(', ')}) #{result.lstrip}" : result
    end

    private

    def option_to_sql(option, value=nil)
      str = " #{option}".upcase.split('_').join(' ')
      value.nil? ? str : str.lstrip + " #{value}"
    end
  end
end
